/****************************************************************************
 * UniPROG Universal programmer 
 * Chip database
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 08/05/2001 - First version
 * 01/09/2003 - Added vendor indentify function
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "chipdb.h"

#define NUM_DEVICES 31

CHIPREG chipdbTable[NUM_DEVICES] = {
  // Old Bipolar Eproms
  { "2716"  ,   0x0000, "2Kx8 UV Eprom",   "Generic", UVEPROM, 26, 2,   8, 0, 5.0, 21.0 },
  { "2732"  ,   0x0000, "4Kx8 UV Eprom",   "Generic", UVEPROM, 26, 4,   8, 0, 5.0, 21.0 },
  { "2764"  ,   0x0000, "8Kx8 UV Eprom",   "Generic", UVEPROM, 28, 8,   8, 0, 5.0, 21.0 },
  // CMOS Eproms
  { "27C64" ,   0x0000, "8Kx8 UV Eprom",   "Generic", UVEPROM, 28, 8,   8, 0, 5.0, 12.5 },
  { "27C256",   0x0000, "32Kx8 UV Eprom",  "Generic", UVEPROM, 28, 32,  8, 0, 5.0, 12.5 },
  { "27C512",   0x0000, "64Kx8 UV Eprom",  "Generic", UVEPROM, 28, 64,  8, 0, 5.0, 12.5 },
  { "27C010",   0x0000, "128Kx8 UV Eprom", "Generic", UVEPROM, 32, 128, 8, 0, 5.0, 12.5 },
  { "27C020",   0x0000, "256Kx8 UV Eprom", "Generic", UVEPROM, 32, 256, 8, 0, 5.0, 12.5 },
  { "27C040",   0x0000, "512Kx8 UV Eprom", "Generic", UVEPROM, 32, 512, 8, 0, 5.0, 12.5 },
  // AMD Flash
  { "am29F512", 0x0100, "64Kx8 FLASH",  "AMD", FLASH, 28, 64,  8, 0, 5.0, 0.0 },
  { "am29F010", 0x0120, "128Kx8 FLASH", "AMD", FLASH, 32, 128, 8, 0, 5.0, 0.0, 8 },
  { "am29F040", 0x01A4, "512Kx8 FLASH", "AMD", FLASH, 32, 512, 8, 0, 5.0, 0.0, 8 },
  // Atmel Sector Flash
  { "AT29C256", 0x1FDC, "32Kx8, 5-Volt Flash",  "ATMEL", FLASH, 28, 32,  8, 0, 5.0, 0.0, 512},
  { "AT29C512", 0x1F5D, "64Kx8, 5-Volt Flash",  "ATMEL", FLASH, 28, 64,  8, 0, 5.0, 0.0, 512},
  { "AT29C010", 0x1FD5, "128Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 128, 8, 0, 5.0, 0.0, 1024},
  { "AT29C020", 0x1FDA, "256Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 256, 8, 0, 5.0, 0.0, 1024},
  { "AT29C040", 0x1FA4, "512Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 512, 8, 0, 5.0, 0.0, 2048},
  // Atmel Flash
  { "AT49F256", 0x0000, "32Kx8, 5-Volt Flash",  "ATMEL", FLASH, 28, 32,  8, 0, 5.0, 0.0},
  { "AT49F512", 0x0000, "64Kx8, 5-Volt Flash",  "ATMEL", FLASH, 28, 64,  8, 0, 5.0, 0.0},
  { "AT49F010", 0x0000, "128Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 128, 8, 0, 5.0, 0.0},
  { "AT49F020", 0x0000, "256Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 256, 8, 0, 5.0, 0.0},
  { "AT49F040", 0x0000, "512Kx8, 5-Volt Flash", "ATMEL", FLASH, 32, 512, 8, 0, 5.0, 0.0},
  // SST FLASH
  { "SST29EE512", 0x0000, "64Kx8, 5-Volt Flash",  "SST", FLASH, 28, 64,  8, 0, 5.0, 0.0 },
  { "SST29EE010", 0x0000, "128Kx8, 5-Volt Flash", "SST", FLASH, 32, 128, 8, 0, 5.0, 0.0 },
  { "SST29EE020", 0x0000, "256Kx8, 5-Volt Flash", "SST", FLASH, 32, 256, 8, 0, 5.0, 0.0 },
  { "SST29EE040", 0x0000, "512Kx8, 5-Volt Flash", "SST", FLASH, 32, 512, 8, 0, 5.0, 0.0 },
  // SST Sector Flash
  { "SST39SF512", 0x0000, "64Kx8, 5-Volt Flash",  "SST", FLASH, 28, 64,  8, 0, 5.0, 0.0 },
  { "SST39SF010", 0x0000, "128Kx8, 5-Volt Flash", "SST", FLASH, 32, 128, 8, 0, 5.0, 0.0 },
  { "SST39SF020", 0x0000, "256Kx8, 5-Volt Flash", "SST", FLASH, 32, 256, 8, 0, 5.0, 0.0 },
  { "SST39SF040", 0x0000, "512Kx8, 5-Volt Flash", "SST", FLASH, 32, 512, 8, 0, 5.0, 0.0 }
  };


// Load chips database
int chipdbInit( void )
  {
  return 0;
  }

// Get Number of devices
int chipdbGetNum( void )
  {
  return NUM_DEVICES;
  }

// Get device n
CHIPREG *chipdbGet( int reg )
  {
  if (reg >= NUM_DEVICES)
    return NULL;
  return &chipdbTable[reg];
  }

// Locate chip in database
CHIPREG *chipdbFind( char *name )
  {
  CHIPREG *p = chipdbTable;
  int i = NUM_DEVICES;

  for (; --i>=0; p++)
    if (!strcmp(p->name, name))
      return p;
  return NULL;
  }

CHIPREG *chipdbFindId( unsigned int id )
  {
  CHIPREG *p = chipdbTable;
  int i = NUM_DEVICES;

  for (; --i>=0; p++)
    if (p->id == id)
      return p;
  return NULL;
  }


void chipdbPrintList( void )
  {
  CHIPREG *p = chipdbTable;
  int i = NUM_DEVICES;

  for (; --i>=0; p++)
    {
    printf("%10s - %20s %10s\n", p->name, p->desc, p->brand);
    }
  }

const char *chipdbGetVendorName(unsigned char id) {
  const char *name;

  switch(id) {
  case 0x1F: name = "ATMEL"; break;
  case 0xDA: name = "ASD/WinBond"; break;
  case 0x20: name = "ST"; 	break;
  case 0x01: name = "AMD"; 	break;
  case 0x31: name = "CATALYST"; break;
  case 0x89: name = "Intel"; 	break;
  case 0xAD: name = "Hyudai"; 	break;
  case 0xBF: name = "SST"; 	break;
  case 0xC2: name = "MXIC"; 	break;
  case 0x04: name = "Fujitsu";	break;
  case 0xB0: name = "SHARP"; 	break;
  case 0x40: name = "SyncMOS"; 	break;
  case 0x42: name = "SyncMOS"; 	break;
  default:   name = "Unknown";
  }

  return name;
}
