/****************************************************************************
 * UniPROG Universal programmer 
 * IntelHex and bynary file support
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 * IntelHex routines by Paul Stoffregen, <paul@ece.orst.edu>
 *
 * 20/05/2001 - First version
 *              TODO: Make IntelHex load routine
 */

#include "stdhdr.h"
#include "intelhex.h"

#define RECORD_SIZE 16


/* Returns the extension of a filename */
char *getFilenameExt(char *filename) 
  {
    char *p = filename;

    while (*p && *p != '.') p++;
     
    if (*p == '.')
      return p+1;
    else
      return p;
  }

char getFilenameFormat(char *filename) 
  {
    char ext[255], *p;

    /* get extension and convert to lowercase */
    strcpy(ext, getFilenameExt(filename));
    for (p = ext; *p; p++)
      *p = tolower(*p);

    if (!strcmp("bin", ext))
      return 'b';
    else if (!strcmp("hex", ext))
      return 'h';
    else
      return '\0';
  }

    
int fileLoad( char *filename, unsigned char *buffer,
              unsigned long buffer_size, unsigned long *size, char format)
  {
  int err;

  switch (format) {
    case 'h':
        err = iHexLoad(filename, buffer, buffer_size, size); break;
    case 'b':
        err = binLoad(filename, buffer, buffer_size, size); break;
    default :
        err = -1;
    }
  return err;
  }

int fileSave( char *filename, unsigned char *buffer,
              unsigned long buffer_size, char format)
  {
  int err;

  switch (format) {
    case 'h':
        err = iHexSave(filename, buffer, buffer_size); break;
    case 'b':
        err = binSave(filename, buffer, buffer_size); break;
    default :
        err = -1;
    }
  return err;
  }


int binSave( char *filename, unsigned char *buffer, unsigned long buffer_size)
  {
  FILE *fich;
  assert(filename);
  assert(buffer);

  // try to create te file
  if ((fich = fopen( filename, "wb" )) == NULL)
    return -1;

  // write data
  fwrite( buffer, buffer_size, sizeof(char), fich);

  // close File
  fclose(fich);

  return 0;
  }

int binLoad( char *filename, unsigned char *buffer,
             unsigned long buffer_size, unsigned long *size)
  {
  unsigned long fsz;
  FILE *fich;
  assert(filename);
  assert(buffer);
  assert(size);

  // Try to open the file
  if ((fich = fopen( filename, "rb")) == NULL)
    return -1;

  // get file size
  fseek(fich, 0, SEEK_END);
  fsz = ftell(fich);
  fseek(fich, 0, SEEK_SET);

  // check if file size enters on buffer
  if (fsz > buffer_size)
    fsz = buffer_size;
  *size = fsz;

  // read all data
  fread( buffer, fsz, sizeof(char), fich);

  // close the file
  fclose(fich);

  return 0;
  }


int iHexLoad( char *filename, unsigned char *buffer,
              unsigned long buffer_size, unsigned long *size)
  {
    printf("IntelHex load routine isnt implemented, use binary format instead\n");

/*  unsigned long addr;
  unsigned char *pData = buffer;
  char cad[20 + RECORD_SIZE*2];
  FILE *fich;
  int i;
  assert(filename);
  assert(buffer);




*/
  return -1;
  }


int iHexSave( char *filename, unsigned char *buffer, unsigned long buffer_size)
  {
  unsigned long addr;
  unsigned char *pData = buffer;
  char cad[20 + RECORD_SIZE*2];
  FILE *fich;
  assert(filename);
  assert(buffer);

  // try to create te file
  if ((fich = fopen( filename, "wt" )) == NULL)
    return -1;

  // write data lines
  for (addr = 0; addr < buffer_size; addr+=RECORD_SIZE, pData+=RECORD_SIZE)
    {
    if (iHexOutDataLine( cad, sizeof(cad), addr, pData,
                         (addr - buffer_size < RECORD_SIZE) ?
                          addr - buffer_size : RECORD_SIZE) == -1)
      {
      fclose(fich);
      return -1;
      }
    fputs(cad, fich);
    }

  // write end file
  iHexOutEndLine( cad, sizeof(cad));
  fputs(cad, fich);
  fclose(fich);

  return 0;
  }


int iHexOutDataLine( char *cad, int sizecad,
                     unsigned long addr, char *data, int size_data )
  {
  unsigned char *pCad = cad, *pData = data;
  char chksum = 0;
  int i;
  assert(cad);

  // check for enought space
  if (sizecad < (13 + (size_data*2)))
    return -1;

  // header
  pCad += sprintf(cad, ":%02X%04lX00", size_data, addr);
  chksum = (size_data & 0xff) + ((addr>>8) & 0xff) + (addr & 0xff);
  // data
  for ( i=0; i < size_data; i++)
    {
    pCad += sprintf(pCad, "%02X", *pData);
    chksum += (*pData++) & 0xff;
    }

  // checksum
  sprintf(pCad, "%02X\n", (-chksum) & 0xff);

  return 0;
  }

int iHexOutEndLine( char *cad, int sizecad )
  {
  assert(cad);

  // check for enought space
  if (sizecad < 13)
    return -1;

  // header
  sprintf(cad, ":00000001FF\n");

  return 0;
  }




