/****************************************************************************
 * UniPROG Universal programmer 
 * Command line parser
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 04/05/2001 - First version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "parseargs.h"

// Search arguments table for a valid one
ARG *argFind( char *pLine, ARG *pArg, int nArg)
  {
  int n, m2;
  char cad[20];

  strcpy(cad, pLine);
  cad[strcspn(cad, ":=")] = '\0';

  for (; --nArg>=0; pArg++)
    {
    char *p = pArg->key;
    n = 0;
    while (*p)
      {
      n++;
      m2 = strcspn(p, ",");
      if (!strncmp(cad, p, m2))
        {
	  /*pArg->used = n;*/ /* disabled for use constant table */ 
        return pArg;
        }
      p += m2;
      if (*p) p++;
      }
    }
  return NULL;
  }

typedef int (*PFI)();

void argSet(ARG *pArg, char *pLine)
  {
  switch (pArg->type) {
    case INTEGER:   *pArg->val = atoi(pLine); break;
    case BOOLEAN:   *pArg->val = 1; break;
    case CHARACTER: *pArg->val = *pLine; break;
    case STRING:    *(char **)pArg->val=pLine; break;
    case PROC:      (*(PFI)(pArg->val))(pLine); break;
    case GROUP:     *pArg->val = pArg->used; break;
    default:
      fprintf(stderr,"INTERNAL ERROR: BAD ARGUMENT TYPE\n"); break;
    }
  }

void argPrintUsage(ARG *pArg, int nArg)
  {
  for(; --nArg>=0; pArg++)
    {
    switch(pArg->type)
      {
      case INTEGER:
      	fprintf(stderr,"  %-8s=<num> \t%-40s\n", pArg->key, pArg->errmsg);
        //fprintf(stderr," (value is %-5d)\n",*(pArg->val));
	break;
      case BOOLEAN:
      case GROUP:
	fprintf(stderr,"  %-8s       \t%-40s\n", pArg->key, pArg->errmsg);
        //fprintf(stderr," (value is %-5s)\n",*(pArg->val) ?"TRUE":"FALSE");
        break;
      case CHARACTER:
	fprintf(stderr,"  %-8s=<c>   \t%-40s\n", pArg->key,pArg->errmsg);
        //fprintf(stderr," (value is %-5c)\n",*(pArg->val));
	break;
      case STRING:
        fprintf(stderr,"  %-8s=<str> \t%-40s\n", pArg->key, pArg->errmsg);
        //fprintf(stderr," (value is \"%s\")\n", *(char **)pArg->val);
        break;
      case PROC:
	fprintf(stderr,"  %-8s=<str> \t%-40s\n", pArg->key,pArg->errmsg);
        break;
      }
    }
  }


#define ERRMSG "Illegal argument \'%c\'.  Legal arguments are:\n\n"

// Analyze the command line, the arguments is defined in argTable
int argParse( int argc, char *argv[], ARG *argTable, int argTableSize )
  {
  char *p;
  ARG  *pArg;

  argv++;
  for (; --argc>0; argv++)      // allways skip exe name (argv[0])
    {
    // pointer to arg string
      //    if ((**argv == '-') || (**argv == '/'))
      //{
      p = *argv;
      if ((pArg = argFind(p, argTable, argTableSize)) != NULL)
        {
        // advance until ':', '=' or null
        while (!((*p == ':') || (*p == '=') || (*p == '\0'))) p++;
        argSet(pArg, ++p);
        }
      else
        {
	fprintf(stderr,ERRMSG,*p);
        argPrintUsage(argTable, argTableSize);
	return 1;
        }
      //}

    }

  return 0;
  }
